# -*- coding: utf-8 -*-
import irc3
import os
import re
import requests

from docopt import Dict as DocOptDict
from irc3 import event, IrcBot
from irc3.plugins.command import command
from irc3.utils import IrcString

from . import Plugin
from ..utils import date_from_iso


# TODO: write better code lol
@irc3.plugin
class YouTube(Plugin):
    requires = ['irc3.plugins.command']

    URL = 'https://www.googleapis.com/youtube/v3'
    API = '{}/videos?part=snippet,statistics,contentDetails'.format(URL)
    SEARCH = '{}/search?part=id,snippet'.format(URL)

    def __init__(self, bot: IrcBot):
        super().__init__(bot)
        self.GOOGLE_API_KEY = os.environ['GOOGLE_API_KEY']

    def get_video_data(self, item: dict):
        date = date_from_iso(item['snippet']['publishedAt'])

        try:
            length = item['contentDetails']['duration']
            length = [l.lower() for l in re.findall('(\d+[DHMS])', length)]
            length = ' - length {}'.format(' '.join(length))
        except KeyError:
            length = ''

        try:
            views = int(item['statistics'].get('viewCount', 0))
            likes = int(item['statistics'].get('likeCount', 0))
            dislikes = int(item['statistics'].get('dislikeCount', 0))

            try:
                score = 100 * float(likes) / (likes + dislikes)
            except ZeroDivisionError:
                score = 0

            stats = ' -\x033 {likes:,}\x0F /\x035 ' \
                    '{dislikes:,}\x0F ({score:,.1f}%) ' \
                    '- {views:,} views'.format(likes=likes,
                                               dislikes=dislikes,
                                               score=score,
                                               views=views)
        except KeyError:
            stats = ''

        return '{title}{length}{stats} - {channel} on {date}'.format(
            title=item['snippet']['title'],
            channel=item['snippet']['channelTitle'],
            length=length,
            date=date.strftime('%Y.%m.%d'),
            stats=stats,
        )

    @event(r'(?i)^:.* PRIVMSG (?P<target>.*) :.*(?:youtube.*?(?:v=|/v/)'
           r'|youtu\.be/)(?P<video_id>[-_a-zA-Z0-9]+).*')
    def youtube_parser(self, target: str, video_id: str):
        req = requests.get(self.API, params={
            'key': self.GOOGLE_API_KEY,
            'id': video_id})
        data = req.json()

        if data['items']:
            self.bot.privmsg(target, self.get_video_data(data['items'][0]))

    @command
    def yt(self, mask: IrcString, channel: IrcString, args: DocOptDict):
        """Searches for query on YouTube and returns first result.

        %%yt <query>...
        """
        req = requests.get(self.SEARCH, params={
            'key': self.GOOGLE_API_KEY,
            'q': ' '.join(args['<query>'])})
        data = req.json()

        if 'error' in data:
            return '[YouTube] Error performing search'
        elif data['pageInfo']['totalResults'] is 0:
            return '[YouTube] No results found'
        else:
            item = data['items'][0]
            return '{response} - https://youtu.be/{video_id}'.format(
                response=self.get_video_data(item),
                video_id=item['id']['videoId'])
